@@ -193,9 +193,17 @@ int dialog_checklist (const char *title, const char *prompt, int height,
 extern unsigned char dialog_input_result[];
 int dialog_inputbox (const char *title, const char *prompt, int height,
 		int width, const char *init);
-int dialog_guage (const char *title, const char *prompt, int height, int width,
+int dialog_gauge (const char *title, const char *prompt, int height, int width,
 		int percent);
 
+void add_atk_namedesc (GtkWidget *widget, const gchar *name, const gchar *desc);
+void add_atk_relation (GtkWidget *obj1, GtkWidget *obj2, AtkRelationType type);
+
+#ifdef WITH_GNOME
+gboolean callback_progress_bar (GIOChannel *giochannel, GIOCondition condition,
+		gpointer p);
+#endif
+
 /*
  * The following stuff is needed for mouse support
  */
