@@ -44,7 +44,9 @@ read_stdinput (gpointer data,
 	char curbuf[BUFSIZE];
 	long int val;
 	GFloppy *floppy = (GFloppy *)data;
+
 	gdk_threads_enter ();
+
 	if (condition != GDK_INPUT_READ)
 		return;
 
@@ -65,6 +67,9 @@ read_stdinput (gpointer data,
 
 			/* found a '\000' so parse message */
 			switch (buf[0]) {
+			case 'B':
+				floppy->nbadblocks++;
+				break;
 			case 'E':
 				show_error_dialog = TRUE;
 				error_label = g_strdup (buf+1);
@@ -104,26 +109,37 @@ read_stdinput (gpointer data,
 		int status;
 		int rc;
 		GtkWidget *error_dialog;
-		GtkWidget *w;
-		
+
 		wait4 (floppy->pid, &status, WNOHANG, NULL);
 		rc = WEXITSTATUS (status);
 
 		/* destroy input handler */
  		gdk_input_remove (floppy->handler_id);
 		if (rc == 0 && !WIFSIGNALED (status)) {
-			error_label = g_strdup (_("Floppy formatted successfully."));
 			show_error_dialog = TRUE;
+	
+			if (floppy->nbadblocks > 0) {
+				dialog_type = GTK_MESSAGE_WARNING;
+				error_label = g_strdup_printf (_("The floppy has been formatted, but <b>%d bad blocks</b> (out of %d) have been found and marked."), floppy->nbadblocks, floppy->nblocks);
+			} else {
+				dialog_type = GTK_MESSAGE_INFO;
+				error_label = g_strdup (_("Floppy formatted successfully."));
+			}
 		}
-		if (show_error_dialog == FALSE)
-			error_label = g_strdup (_("Floppy formatting cancelled."));
-					    
 
-		error_dialog = gtk_message_dialog_new ((GtkWindow *)progress_dialog, 0,
+		if (show_error_dialog == FALSE) {
+			dialog_type = GTK_MESSAGE_INFO;
+			error_label = g_strdup (_("Floppy formatting cancelled."));
+		}
+
+		error_dialog = gtk_message_dialog_new ((GtkWindow *)progress_dialog, GTK_DIALOG_MODAL,
 						       dialog_type,
 						       GTK_BUTTONS_CLOSE,
 						       error_label);
 
+		if (rc == 0 && !WIFSIGNALED (status) && floppy->nbadblocks > 0)
+                        gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (error_dialog)->label), TRUE);
+
 		/* reset the dialog properties */
 		dialog_type = GTK_MESSAGE_INFO;
 		g_free (error_label);
@@ -136,46 +152,51 @@ read_stdinput (gpointer data,
 	gdk_threads_leave ();
 }
 
-static void
-progress_close_callback (GtkWidget *progress, gpointer data)
-{
-	GFloppy *floppy = (GFloppy *) data;
-
-	kill (floppy->pid, SIGTERM);
-}
-
 static void
 progress_response_callback (GtkWidget *progress, gint response, gpointer data)
 {
 	GFloppy *floppy = (GFloppy *) data;
 
 	kill (floppy->pid, SIGTERM);
+
+	if (response == GTK_RESPONSE_DELETE_EVENT)
+		progress_dialog = NULL; /* so the dialog isn't destroyed twice */
 }
 
 void
 setup_progress_and_run (GFloppy *floppy, GtkWidget *parent)
 {
+	GtkWidget *vbox;
 
 	show_error_dialog = FALSE;
-	progress_dialog = gtk_dialog_new_with_buttons (_("Format progress."),
+	progress_dialog = gtk_dialog_new_with_buttons (_("Format progress"),
 						       GTK_WINDOW (parent),
-						       0,
+						       GTK_DIALOG_MODAL,
 						       GTK_STOCK_CANCEL,
 						       GTK_RESPONSE_CANCEL,
 						       NULL);
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 7);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (progress_dialog)->vbox), vbox,
+			    TRUE, TRUE, 0);
+
 	msg_label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (msg_label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (progress_dialog)->vbox), msg_label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), msg_label, FALSE, FALSE, 0);
 	gtk_widget_show (msg_label);
 	progressbar = gtk_progress_bar_new ();
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (progress_dialog)->vbox), progressbar, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), progressbar, FALSE, FALSE, 0);
+
 	floppy->handler_id = gdk_input_add (floppy->message[0], GDK_INPUT_READ, read_stdinput, floppy);
 
-	gtk_signal_connect (GTK_OBJECT (progress_dialog), "close", GTK_SIGNAL_FUNC (progress_close_callback), floppy);
 	gtk_signal_connect (GTK_OBJECT (progress_dialog), "response", GTK_SIGNAL_FUNC (progress_response_callback), floppy);
 
 	gtk_widget_show_all (progress_dialog);
+
 	gtk_main ();
-	gtk_widget_destroy (progress_dialog);
-	progress_dialog = NULL;
+
+	if (progress_dialog) {
+		gtk_widget_destroy (progress_dialog);
+		progress_dialog = NULL;
+	}
 }
